import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import japanize_matplotlib
from scipy.cluster.hierarchy import linkage
from scipy.cluster.hierarchy import dendrogram
from scipy.cluster.vq import kmeans, whiten

#   ExcelファイルをPandasのDataFrame型として読み込む
fl_nm = 'rain.xlsx' 
with pd.ExcelFile(fl_nm) as xlfile:
    df = pd.read_excel(xlfile)

cities = df.values[:,0]
data_v = np.array(df.values[:,1:], dtype=float)
data_wh = whiten(data_v)

init_c = {'新潟', '山形', '東京', '静岡', '長崎'}
k = 5
init_cities = []
init_coords = []
for i in range(len(cities)):
    if cities[i] in init_c:
        init_cities.append(cities[i])
        init_coords.append(data_wh[i])
init_coords = np.array(init_coords)
print('init_clusters,,,\n')
for v in zip(init_cities, init_coords):
    print('city =', v[0], '  coord =', v[1])
#
#          k-meansクラスター分析 
#
results = kmeans(data_wh, init_coords)
#
#         描画の準備のための固有分解
#
Z = data_wh - np.mean(data_wh, axis=0)
vals, vecs = np.linalg.eigh(Z.T @ Z)
#       固有値の大きさの降順に並べ換える
idx = np.argsort(-vals)
vals = vals[idx]
vecs = vecs[:,idx]
#
#　　　　固有値のスクリープロット
#
plt.plot(range(len(vals)), vals)
plt.title('Eigen Values', fontsize=16)
plt.tight_layout()
plt.show()
#
#        直交回転
#
data_wh_rot = data_wh @ vecs
k_means_rot = results[0] @ vecs
#
#       対象と各クラスター中心までの距離
#
d_obj_clus = np.empty((len(cities), k))
for i in range(len(cities)):
    for j in range(k):
        d_obj_clus[i][j] = (np.sum((data_wh_rot[i] - k_means_rot[j])**2))**0.5
#     対象から最小距離のクラスター
obj_clus = np.empty(len(cities))
for i in range(len(cities)):
    obj_clus[i] = np.argmin(d_obj_clus[i])
#
#        ウエイトのグラフ
#
for i in range(len(vecs)):
    plt.plot(range(len(vecs.T)), vecs.T[i], label=f'w[{i+1}]')
plt.title('Weights for the Rotation', fontsize=16)
plt.legend()
plt.tight_layout()
plt.show()
#
#         対象とクラスター中心の散布図
#
dim_x = 1
dim_y = 2
while True:
    plt.plot(k_means_rot[:,dim_x-1], k_means_rot[:,dim_y-1], linewidth=0,
             marker = '*', markersize=25, alpha=0.7, label='Cluster')
    plt.plot(data_wh_rot[:,dim_x-1], data_wh_rot[:,dim_y-1], linewidth=0,
             marker='o', label='Object')
    for i in range(len(cities)):
        plt.text(data_wh_rot[i,dim_x-1], data_wh_rot[i,dim_y-1],
                 f'{int(obj_clus[i])+1}'+ cities[i])
    plt.xlabel(f'Axis-{dim_x}')
    plt.ylabel(f'Axis-{dim_y}')
    plt.legend()
    plt.show()

    print('Input the dimensions, larger than 0 and smaller than' +
            f' {len(data_wh[0])}.')
    dim_x = int(input('dim_x = '))
    if (dim_x < 1) or (dim_x > len(data_wh[0])):
        break
    dim_y = int(input('dim_y = '))
    if (dim_y < 1) or (dim_y > len(data_wh[0])):
        break

