import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import japanize_matplotlib
from scipy.cluster.vq import kmeans, whiten

#   ExcelファイルをPandasのDataFrame型として読み込む
fl_nm = 'rain.xlsx' 
with pd.ExcelFile(fl_nm) as xlfile:
    df = pd.read_excel(xlfile)

data_v = np.array(df.values[:,1:], dtype=float)
#       Whitening
data_wh = whiten(data_v)
#
#      各月の平均値と標準偏差
#
mean_raw = np.mean(data_v, axis=0)
sd_raw = np.std(data_v, axis=0)
mean_w = np.mean(data_wh, axis=0)
sd_w = np.std(data_wh, axis=0)

print('Comparison between raw data and whitened data')
for i in range(12):
    print(f'{i+1}月:')
    print('Before: Mean =', mean_raw[i], '   SD =', sd_raw[i])
    print('After:  Mean =', mean_w[i], '   SD =', sd_w[i])
    print('mean_raw/sd_raw =', mean_raw[i]/sd_raw[i])

max_v = np.max(data_v)
print('max_v =', max_v)
#
#      ヒストグラムの境界値の設定
#
v_bins = np.arange(0, 450, 25)
fig, ax = plt.subplots(3, 4, sharex=True, sharey=True, figsize=(10,7))
for i in range(12):
    r_idx = i // 4 
    c_idx = i % 4 
    ax[r_idx, c_idx].hist(data_v.T[i], bins=v_bins)
    ax[r_idx, c_idx].set_title(f'{i+1}月(SD={sd_raw[i]:.2f})')
    ax[r_idx, c_idx].set_xlabel('mm')
    ax[r_idx, c_idx].set_ylabel('度\n\n数', rotation=0, va='center')
fig.suptitle('平年降水量', fontsize=18)
plt.tight_layout()
plt.show()

max_vw = np.max(data_wh)
print('max_vw =', max_vw)
#
#       ヒストグラムの境界値の設定
#
v_bins_w = np.arange(0, 8, 0.5)
fig, ax = plt.subplots(3, 4, sharex=True, sharey=True, figsize=(10,7))
for i in range(12):
    r_idx = i // 4 
    c_idx = i % 4 
    ax[r_idx, c_idx].hist(data_wh.T[i], bins=v_bins_w)
    ax[r_idx, c_idx].set_title(f'{i+1}月(SD={sd_w[i]:.2f})')
    ax[r_idx, c_idx].set_xlabel('SD')
    ax[r_idx, c_idx].set_ylabel('度\n\n数', rotation=0, va='center')
fig.suptitle('平年降水量(Whitened)', fontsize=18)
plt.tight_layout()
plt.show()
